"""
    Gather all full pipelines

"""
import nipype.interfaces.utility as niu
import nipype.pipeline.engine as pe

from nipype.interfaces.fsl.maths import (
    DilateImage, ErodeImage,
    ApplyMask, UnaryMaths, Threshold)

from nipype.interfaces.fsl.utils import RobustFOV
from nipype.interfaces.fsl.preprocess import FAST, FLIRT


from nipype.interfaces.niftyreg.regutils import RegResample
from nipype.interfaces.niftyreg.reg import RegAladin

from macapype.utils.utils_nodes import NodeParams

from nipype.interfaces.ants import N4BiasFieldCorrection

from macapype.pipelines.prepare import _create_avg_reorient_pipeline

from macapype.nodes.prepare import average_align

from macapype.nodes.surface import (keep_gcc, wrap_afni_IsoSurface)

from skullTo3d.nodes.skull import (
    mask_auto_img)

from macapype.utils.misc import parse_key, get_elem

#################################################
# ####################  T1  #####################
#################################################


def create_skull_t1_pipe(name="skull_t1_pipe", params={}):

    # Creating pipeline
    skull_t1_pipe = pe.Workflow(name=name)

    # Creating input node
    inputnode = pe.Node(
        niu.IdentityInterface(fields=['indiv_params', 'stereo_native_T1']),
        name='inputnode')

    # ### head mask
    # headmask_threshold
    if "t1_head_mask_thr" in params.keys():
        # t1_head_mask_thr
        t1_head_mask_thr = NodeParams(
            interface=Threshold(),
            params=parse_key(params, 't1_head_mask_thr'),
            name="t1_head_mask_thr")

        skull_t1_pipe.connect(inputnode, "stereo_native_T1",
                              t1_head_mask_thr, "in_file")

        skull_t1_pipe.connect(
            inputnode, ('indiv_params', parse_key, "t1_head_mask_thr"),
            t1_head_mask_thr, "indiv_params")
    else:

        t1_head_auto_mask = NodeParams(
                interface=niu.Function(
                    input_names=["img_file", "operation",
                                 "index", "sample_bins", "distance", "kmeans"],
                    output_names=["mask_img_file"],
                    function=mask_auto_img),
                params=parse_key(params, "t1_head_auto_mask"),
                name="t1_head_auto_mask")

        skull_t1_pipe.connect(inputnode, "stereo_native_T1",
                              t1_head_auto_mask, "img_file")

        skull_t1_pipe.connect(
            inputnode, ('indiv_params', parse_key, "t1_head_auto_mask"),
            t1_head_auto_mask, "indiv_params")

    # t1_head_mask_binary
    t1_head_mask_binary = pe.Node(interface=UnaryMaths(),
                                  name="t1_head_mask_binary")

    t1_head_mask_binary.inputs.operation = 'bin'
    t1_head_mask_binary.inputs.output_type = 'NIFTI_GZ'

    if "t1_head_mask_thr" in params.keys():

        skull_t1_pipe.connect(t1_head_mask_thr, "out_file",
                              t1_head_mask_binary, "in_file")
    else:

        skull_t1_pipe.connect(t1_head_auto_mask, "mask_img_file",
                              t1_head_mask_binary, "in_file")

    # keep_gcc_t1_head
    keep_gcc_t1_head = pe.Node(
        interface=niu.Function(input_names=["nii_file"],
                               output_names=["gcc_nii_file"],
                               function=keep_gcc),
        name="keep_gcc_t1_head")

    skull_t1_pipe.connect(t1_head_mask_binary, "out_file",
                          keep_gcc_t1_head, "nii_file")

    # t1_head_dilate
    t1_head_dilate = NodeParams(interface=DilateImage(),
                                params=parse_key(params, "t1_head_dilate"),
                                name="t1_head_dilate")

    skull_t1_pipe.connect(keep_gcc_t1_head, "gcc_nii_file",
                          t1_head_dilate, "in_file")

    skull_t1_pipe.connect(
        inputnode, ('indiv_params', parse_key, "t1_head_dilate"),
        t1_head_dilate, "indiv_params")

    # t1_head_fill
    t1_head_fill = pe.Node(interface=UnaryMaths(),
                           name="t1_head_fill")

    t1_head_fill.inputs.operation = 'fillh'

    skull_t1_pipe.connect(t1_head_dilate, "out_file",
                          t1_head_fill, "in_file")

    # t1_head_erode
    t1_head_erode = NodeParams(interface=ErodeImage(),
                               params=parse_key(params, "t1_head_erode"),
                               name="t1_head_erode")

    skull_t1_pipe.connect(t1_head_fill, "out_file",
                          t1_head_erode, "in_file")

    skull_t1_pipe.connect(
        inputnode, ('indiv_params', parse_key, "t1_head_erode"),
        t1_head_erode, "indiv_params")

    # t1_hmasked
    t1_hmasked = pe.Node(interface=ApplyMask(),
                         name="t1_hmasked")

    skull_t1_pipe.connect(inputnode, "stereo_native_T1",
                          t1_hmasked, "in_file")

    skull_t1_pipe.connect(t1_head_erode, "out_file",
                          t1_hmasked, "mask_file")

    if "t1_debias" in params.keys():

        # N4 intensity normalization over T1
        t1_debias = NodeParams(N4BiasFieldCorrection(),
                               params=parse_key(params, "t1_debias"),
                               name='t1_debias')

        skull_t1_pipe.connect(t1_hmasked, "out_file",
                              t1_debias, "input_image")

        skull_t1_pipe.connect(
            inputnode, ('indiv_params', parse_key, "t1_debias"),
            t1_debias, "indiv_params")

    t1_fast = NodeParams(interface=FAST(),
                         params=parse_key(params, "t1_fast"),
                         name="t1_fast")

    if "t1_debias" in params.keys():
        skull_t1_pipe.connect(t1_debias, "output_image",
                              t1_fast, "in_files")
    else:
        skull_t1_pipe.connect(t1_hmasked, "out_file",
                              t1_fast, "in_files")

    skull_t1_pipe.connect(
        inputnode, ('indiv_params', parse_key, "t1_fast"),
        t1_fast, "indiv_params")

    if "t1_skull_auto_mask" in params.keys():

        print("*** t1_skull_auto_mask ***")

        t1_skull_auto_mask = NodeParams(
                interface=niu.Function(
                    input_names=["img_file", "operation",
                                 "index", "sample_bins", "distance", "kmeans"],
                    output_names=["mask_img_file"],
                    function=mask_auto_img),
                params=parse_key(params, "t1_skull_auto_mask"),
                name="t1_skull_auto_mask")

        skull_t1_pipe.connect(t1_fast, "restored_image",
                              t1_skull_auto_mask, "img_file")

    # t1_skull_mask_binary
    t1_skull_mask_binary = pe.Node(interface=UnaryMaths(),
                                   name="t1_skull_mask_binary")

    t1_skull_mask_binary.inputs.operation = 'bin'
    t1_skull_mask_binary.inputs.output_type = 'NIFTI_GZ'

    if "t1_skull_auto_mask" in params.keys():
        skull_t1_pipe.connect(t1_skull_auto_mask, "mask_img_file",
                              t1_skull_mask_binary, "in_file")
    else:
        skull_t1_pipe.connect(t1_fast,
                              ("partial_volume_files", get_elem, 0),
                              t1_skull_mask_binary, "in_file")

    # t1_head_erode_skin
    if "t1_head_erode_skin" in params.keys():

        t1_head_erode_skin = NodeParams(
            interface=ErodeImage(),
            params=parse_key(params, "t1_head_erode_skin"),
            name="t1_head_erode_skin")

        skull_t1_pipe.connect(t1_head_erode, "out_file",
                              t1_head_erode_skin, "in_file")

        skull_t1_pipe.connect(
            inputnode, ('indiv_params', parse_key, "t1_head_erode_skin"),
            t1_head_erode_skin, "indiv_params")

        # ### Masking with t1_head mask
        # t1_head_hmasked ####### [okey]
        t1_head_skin_masked = pe.Node(interface=ApplyMask(),
                                      name="t1_head_skin_masked")

        skull_t1_pipe.connect(t1_skull_mask_binary, "out_file",
                              t1_head_skin_masked, "in_file")

        skull_t1_pipe.connect(t1_head_erode_skin, "out_file",
                              t1_head_skin_masked, "mask_file")

    # t1_skull_gcc ####### [okey]
    t1_skull_gcc = pe.Node(
        interface=niu.Function(
            input_names=["nii_file"],
            output_names=["gcc_nii_file"],
            function=keep_gcc),
        name="t1_skull_gcc")

    if "t1_head_erode_skin" in params.keys():

        skull_t1_pipe.connect(t1_head_skin_masked, "out_file",
                              t1_skull_gcc, "nii_file")
    else:
        skull_t1_pipe.connect(t1_skull_mask_binary, "out_file",
                              t1_skull_gcc, "nii_file")

    # t1_skull_dilate
    t1_skull_dilate = NodeParams(
        interface=DilateImage(),
        params=parse_key(params, "t1_skull_dilate"),
        name="t1_skull_dilate")

    skull_t1_pipe.connect(t1_skull_gcc, "gcc_nii_file",
                          t1_skull_dilate, "in_file")

    # t1_skull_t1_fill
    t1_skull_fill = pe.Node(interface=UnaryMaths(),
                            name="t1_skull_fill")

    t1_skull_fill.inputs.operation = 'fillh'

    skull_t1_pipe.connect(t1_skull_dilate, "out_file",
                          t1_skull_fill, "in_file")

    # t1_skull_t1_erode
    t1_skull_erode = NodeParams(interface=ErodeImage(),
                                params=parse_key(params, "t1_skull_erode"),
                                name="t1_skull_erode")

    skull_t1_pipe.connect(t1_skull_fill, "out_file",
                          t1_skull_erode, "in_file")

    # mesh_t1_skull_t1 #######
    mesh_t1_skull_t1 = pe.Node(
        interface=niu.Function(input_names=["nii_file"],
                               output_names=["stl_file"],
                               function=wrap_afni_IsoSurface),
        name="mesh_t1_skull_t1")

    skull_t1_pipe.connect(t1_skull_erode, "out_file",
                          mesh_t1_skull_t1, "nii_file")

    # creating outputnode #######
    outputnode = pe.Node(
        niu.IdentityInterface(
            fields=["t1_skull_mask", "t1_skull_stl", "t1_head_mask"]),
        name='outputnode')

    skull_t1_pipe.connect(t1_head_erode, "out_file",
                          outputnode, "t1_head_mask")

    skull_t1_pipe.connect(mesh_t1_skull_t1, "stl_file",
                          outputnode, "t1_skull_stl")

    skull_t1_pipe.connect(t1_skull_erode, "out_file",
                          outputnode, "t1_skull_mask")

    return skull_t1_pipe

###############################################################################
# #################### CT  ######################
###############################################################################


def create_skull_ct_pipe(name="skull_ct_pipe", params={}):

    # Creating pipeline
    skull_ct_pipe = pe.Workflow(name=name)

    # Creating input node
    inputnode = pe.Node(
        niu.IdentityInterface(fields=['ct', 'stereo_native_T1', 'native_T1',
                                      'native_T2', 'native_to_stereo_trans',
                                      'stereo_native_T1', 'indiv_params']),
        name='inputnode'
    )

    # align_ct_on_T1
    align_ct_on_T1 = pe.Node(interface=RegAladin(),
                             name="align_ct_on_T1")

    skull_ct_pipe.connect(inputnode, 'ct',
                          align_ct_on_T1, "flo_file")

    skull_ct_pipe.connect(inputnode, "native_T1",
                          align_ct_on_T1, "ref_file")

    # align_ct_on_stereo_native_T1
    align_ct_on_stereo_native_T1 = pe.Node(interface=RegResample(pad_val=0.0),
                                           name="align_ct_on_stereo_native_T1")

    skull_ct_pipe.connect(align_ct_on_T1, 'res_file',
                          align_ct_on_stereo_native_T1, "flo_file")

    skull_ct_pipe.connect(inputnode, 'native_to_stereo_trans',
                          align_ct_on_stereo_native_T1, "trans_file")

    skull_ct_pipe.connect(inputnode, "stereo_native_T1",
                          align_ct_on_stereo_native_T1, "ref_file")

    # ct_skull_auto_thresh
    if "ct_skull_mask_thr" in params.keys():

        print("*** ct_skull_mask_thr ***")

        # ct_skull_mask_thr ####### [okey][json]
        ct_skull_mask_thr = NodeParams(
            interface=Threshold(),
            params=parse_key(params, "ct_skull_mask_thr"),
            name="ct_skull_mask_thr")

        ct_skull_mask_thr.inputs.direction = 'above'

        skull_ct_pipe.connect(
            inputnode, ("indiv_params", parse_key, "ct_skull_mask_thr"),
            ct_skull_mask_thr, "indiv_params")

        skull_ct_pipe.connect(align_ct_on_stereo_native_T1, "out_file",
                              ct_skull_mask_thr, "in_file")
    else:

        print("*** ct_skull_auto_mask ***")

        ct_skull_auto_mask = NodeParams(
                interface=niu.Function(
                    input_names=["img_file", "operation",
                                 "index", "sample_bins", "distance", "kmeans"],
                    output_names=["mask_img_file"],
                    function=mask_auto_img),
                params=parse_key(params, "ct_skull_auto_mask"),
                name="ct_skull_auto_mask")

        skull_ct_pipe.connect(align_ct_on_stereo_native_T1, "out_file",
                              ct_skull_auto_mask, "img_file")

        skull_ct_pipe.connect(
            inputnode, ("indiv_params", parse_key, "ct_skull_auto_mask"),
            ct_skull_auto_mask, "indiv_params")

    # ct_skull_mask_binary
    ct_skull_mask_binary = pe.Node(interface=UnaryMaths(),
                                   name="ct_skull_mask_binary")

    ct_skull_mask_binary.inputs.operation = 'bin'
    ct_skull_mask_binary.inputs.output_type = 'NIFTI_GZ'

    if "ct_skull_mask_thr" in params.keys():

        skull_ct_pipe.connect(ct_skull_mask_thr, "out_file",
                              ct_skull_mask_binary, "in_file")
    else:

        skull_ct_pipe.connect(ct_skull_auto_mask, "mask_img_file",
                              ct_skull_mask_binary, "in_file")

    # ct_skull_gcc ####### [okey]
    ct_skull_gcc = pe.Node(
        interface=niu.Function(
            input_names=["nii_file"],
            output_names=["gcc_nii_file"],
            function=keep_gcc),
        name="ct_skull_gcc")

    skull_ct_pipe.connect(ct_skull_mask_binary, "out_file",
                          ct_skull_gcc, "nii_file")

    # ct_skull_dilate ####### [okey][json]
    ct_skull_dilate = NodeParams(
        interface=DilateImage(),
        params=parse_key(params, "ct_skull_dilate"),
        name="ct_skull_dilate")

    skull_ct_pipe.connect(ct_skull_gcc, "gcc_nii_file",
                          ct_skull_dilate, "in_file")

    # ct_skull_fill #######  [okey]
    ct_skull_fill = pe.Node(interface=UnaryMaths(),
                            name="ct_skull_fill")

    ct_skull_fill.inputs.operation = 'fillh'

    skull_ct_pipe.connect(ct_skull_dilate, "out_file",
                          ct_skull_fill, "in_file")

    # ct_skull_erode ####### [okey][json]
    ct_skull_erode = NodeParams(interface=ErodeImage(),
                                params=parse_key(params, "ct_skull_erode"),
                                name="ct_skull_erode")

    skull_ct_pipe.connect(ct_skull_fill, "out_file",
                          ct_skull_erode, "in_file")

    # mesh_ct_skull #######
    mesh_ct_skull = pe.Node(
        interface=niu.Function(input_names=["nii_file"],
                               output_names=["stl_file"],
                               function=wrap_afni_IsoSurface),
        name="mesh_ct_skull")

    skull_ct_pipe.connect(ct_skull_erode, "out_file",
                          mesh_ct_skull, "nii_file")

    # creating outputnode #######
    outputnode = pe.Node(
        niu.IdentityInterface(
            fields=["stereo_ct_skull_mask",
                    "ct_skull_stl"]),
        name='outputnode')

    skull_ct_pipe.connect(mesh_ct_skull, "stl_file",
                          outputnode, "ct_skull_stl")

    skull_ct_pipe.connect(ct_skull_erode, "out_file",
                          outputnode, "stereo_ct_skull_mask")

    return skull_ct_pipe

###############################################################################
# ####################  PETRA  #####################
###############################################################################


def create_skull_petra_pipe(name="skull_petra_pipe", params={}):

    # creating pipeline
    skull_petra_pipe = pe.Workflow(name=name)

    # Creating input node
    inputnode = pe.Node(
        niu.IdentityInterface(fields=['petra', 'stereo_native_T1',
                                      'native_img',
                                      'native_to_stereo_trans',
                                      'indiv_params']),
        name='inputnode'
    )

    # average if multiple PETRA
    if "avg_reorient_pipe" in params.keys():
        print("Found avg_reorient_pipe for av_PETRA")

        av_PETRA = _create_avg_reorient_pipeline(
            name="av_PETRA", params=parse_key(params, "avg_reorient_pipe"))

        skull_petra_pipe.connect(inputnode, 'petra',
                                 av_PETRA, "inputnode.list_img")

        skull_petra_pipe.connect(inputnode, 'indiv_params',
                                 av_PETRA, "inputnode.indiv_params")
    else:
        print("Using average_align for av_PETRA")
        print(params)

        av_PETRA = pe.Node(
            niu.Function(input_names=['list_img', "reorient"],
                         output_names=['avg_img'],
                         function=average_align),
            name="av_PETRA")

        skull_petra_pipe.connect(inputnode, 'petra',
                                 av_PETRA, "list_img")

    # align_petra_on_native
    align_petra_on_native = pe.Node(interface=FLIRT(),
                                    name="align_petra_on_native")

    align_petra_on_native.inputs.apply_xfm = True
    align_petra_on_native.inputs.uses_qform = True
    align_petra_on_native.inputs.interp = 'spline'

    if "avg_reorient_pipe" in params.keys():
        skull_petra_pipe.connect(av_PETRA, 'outputnode.std_img',
                                 align_petra_on_native, "in_file")
    else:
        skull_petra_pipe.connect(av_PETRA, 'avg_img',
                                 align_petra_on_native, "in_file")

    skull_petra_pipe.connect(inputnode, "native_img",
                             align_petra_on_native, "reference")

    # align_petra_on_stereo_native_T1
    align_petra_on_stereo_native_T1 = pe.Node(
        interface=RegResample(pad_val=0.0),
        name="align_petra_on_stereo_native_T1")

    skull_petra_pipe.connect(align_petra_on_native, 'out_file',
                             align_petra_on_stereo_native_T1, "flo_file")

    skull_petra_pipe.connect(inputnode, 'native_to_stereo_trans',
                             align_petra_on_stereo_native_T1, "trans_file")

    skull_petra_pipe.connect(inputnode, "stereo_native_T1",
                             align_petra_on_stereo_native_T1, "ref_file")

    # ### head mask
    # headmask_threshold
    if "petra_head_mask_thr" in params.keys():
        # petra_head_mask_thr
        petra_head_mask_thr = NodeParams(
            interface=Threshold(),
            params=parse_key(params, 'petra_head_mask_thr'),
            name="petra_head_mask_thr")

        skull_petra_pipe.connect(align_petra_on_stereo_native_T1, "out_file",
                                 petra_head_mask_thr, "in_file")

        skull_petra_pipe.connect(
            inputnode, ('indiv_params', parse_key, "petra_head_mask_thr"),
            petra_head_mask_thr, "indiv_params")
    else:

        petra_head_auto_mask = NodeParams(
                interface=niu.Function(
                    input_names=["img_file", "operation",
                                 "index", "sample_bins", "distance", "kmeans"],
                    output_names=["mask_img_file"],
                    function=mask_auto_img),
                params=parse_key(params, "petra_head_auto_mask"),
                name="petra_head_auto_mask")

        skull_petra_pipe.connect(align_petra_on_stereo_native_T1, "out_file",
                                 petra_head_auto_mask, "img_file")

        skull_petra_pipe.connect(
            inputnode, ('indiv_params', parse_key, "petra_head_auto_mask"),
            petra_head_auto_mask, "indiv_params")

    # petra_head_mask_binary
    petra_head_mask_binary = pe.Node(interface=UnaryMaths(),
                                     name="petra_head_mask_binary")

    petra_head_mask_binary.inputs.operation = 'bin'
    petra_head_mask_binary.inputs.output_type = 'NIFTI_GZ'

    if "petra_head_mask_thr" in params.keys():

        skull_petra_pipe.connect(petra_head_mask_thr, "out_file",
                                 petra_head_mask_binary, "in_file")
    else:

        skull_petra_pipe.connect(petra_head_auto_mask, "mask_img_file",
                                 petra_head_mask_binary, "in_file")
    # petra_head_mask_binary_clean1
    petra_head_gcc = pe.Node(
        interface=niu.Function(input_names=["nii_file"],
                               output_names=["gcc_nii_file"],
                               function=keep_gcc),
        name="petra_head_gcc")

    skull_petra_pipe.connect(petra_head_mask_binary, "out_file",
                             petra_head_gcc, "nii_file")

    # petra_head_dilate
    petra_head_dilate = NodeParams(
        interface=DilateImage(),
        params=parse_key(params, "petra_head_dilate"),
        name="petra_head_dilate")

    skull_petra_pipe.connect(petra_head_gcc, "gcc_nii_file",
                             petra_head_dilate, "in_file")

    skull_petra_pipe.connect(
        inputnode, ('indiv_params', parse_key, "petra_head_dilate"),
        petra_head_dilate, "indiv_params")

    # petra_head_fill
    petra_head_fill = pe.Node(interface=UnaryMaths(),
                              name="petra_head_fill")

    petra_head_fill.inputs.operation = 'fillh'

    skull_petra_pipe.connect(petra_head_dilate, "out_file",
                             petra_head_fill, "in_file")

    # petra_head_erode ####### [okey][json]
    petra_head_erode = NodeParams(interface=ErodeImage(),
                                  params=parse_key(params, "petra_head_erode"),
                                  name="petra_head_erode")

    skull_petra_pipe.connect(petra_head_fill, "out_file",
                             petra_head_erode, "in_file")

    skull_petra_pipe.connect(
        inputnode, ('indiv_params', parse_key, "petra_head_erode"),
        petra_head_erode, "indiv_params")

    # ### Masking with head mask
    # petra_hmasked ####### [okey]
    petra_hmasked = pe.Node(interface=ApplyMask(),
                            name="petra_hmasked")

    skull_petra_pipe.connect(align_petra_on_stereo_native_T1, "out_file",
                             petra_hmasked, "in_file")

    skull_petra_pipe.connect(petra_head_erode, "out_file",
                             petra_hmasked, "mask_file")

    # petra_fast
    if "petra_fast" in params.keys():
        petra_fast = NodeParams(interface=FAST(),
                                params=parse_key(params, "petra_fast"),
                                name="petra_fast")

        skull_petra_pipe.connect(petra_hmasked, "out_file",
                                 petra_fast, "in_files")

        skull_petra_pipe.connect(
            inputnode, ('indiv_params', parse_key, "petra_fast"),
            petra_fast, "indiv_params")

    elif "petra_debias" in params.keys():
        # N4 intensity normalization over T1
        petra_debias = NodeParams(N4BiasFieldCorrection(),
                                  params=parse_key(params, "petra_debias"),
                                  name='petra_debias')

        skull_petra_pipe.connect(petra_hmasked, "out_file",
                                 petra_debias, "input_image")

        skull_petra_pipe.connect(
            inputnode, ('indiv_params', parse_key, "petra_debias"),
            petra_debias, "indiv_params")
    else:
        print("Warning, no debias was applied to petra")

    if "petra_skull_auto_mask" in params.keys():

        print("*** petra_skull_auto_mask ***")

        petra_skull_auto_mask = NodeParams(
                interface=niu.Function(
                    input_names=["img_file", "operation",
                                 "index", "sample_bins", "distance", "kmeans"],
                    output_names=["mask_img_file"],
                    function=mask_auto_img),
                params=parse_key(params, "petra_skull_auto_mask"),
                name="petra_skull_auto_mask")

        skull_petra_pipe.connect(petra_fast, "restored_image",
                                 petra_skull_auto_mask, "img_file")

    # petra_skull_mask_binary
    petra_skull_mask_binary = pe.Node(interface=UnaryMaths(),
                                      name="petra_skull_mask_binary")

    petra_skull_mask_binary.inputs.operation = 'bin'
    petra_skull_mask_binary.inputs.output_type = 'NIFTI_GZ'

    if "petra_skull_auto_mask" in params.keys():
        skull_petra_pipe.connect(petra_skull_auto_mask, "mask_img_file",
                                 petra_skull_mask_binary, "in_file")
    else:
        skull_petra_pipe.connect(petra_fast,
                                 ("partial_volume_files", get_elem, 0),
                                 petra_skull_mask_binary, "in_file")

    # petra_skull_auto_thresh
    if "petra_head_erode_skin" in params.keys():

        petra_head_erode_skin = NodeParams(
            interface=ErodeImage(),
            params=parse_key(params, "petra_head_erode_skin"),
            name="petra_head_erode_skin")

        skull_petra_pipe.connect(petra_head_erode, "out_file",
                                 petra_head_erode_skin, "in_file")

        skull_petra_pipe.connect(
            inputnode, ("indiv_params", parse_key, "petra_head_erode_skin"),
            petra_head_erode_skin, "indiv_params")

        # ### Masking with petra_head mask
        # petra_hmasked ####### [okey]
        petra_skin_masked = pe.Node(interface=ApplyMask(),
                                    name="petra_skin_masked")

        skull_petra_pipe.connect(petra_skull_mask_binary, "out_file",
                                 petra_skin_masked, "in_file")

        skull_petra_pipe.connect(petra_head_erode_skin, "out_file",
                                 petra_skin_masked, "mask_file")

    # petra_skull_gcc ####### [okey]
    petra_skull_gcc = pe.Node(
        interface=niu.Function(
            input_names=["nii_file"],
            output_names=["gcc_nii_file"],
            function=keep_gcc),
        name="petra_skull_gcc")

    if "petra_head_erode_skin" in params.keys():

        skull_petra_pipe.connect(petra_skin_masked, "out_file",
                                 petra_skull_gcc, "nii_file")
    else:
        skull_petra_pipe.connect(petra_skull_mask_binary, "out_file",
                                 petra_skull_gcc, "nii_file")

    # petra_skull_dilate ####### [okey][json]
    petra_skull_dilate = NodeParams(
        interface=DilateImage(),
        params=parse_key(params, "petra_skull_dilate"),
        name="petra_skull_dilate")

    skull_petra_pipe.connect(petra_skull_gcc, "gcc_nii_file",
                             petra_skull_dilate, "in_file")

    # petra_skull_fill #######  [okey]
    petra_skull_fill = pe.Node(interface=UnaryMaths(),
                               name="petra_skull_fill")

    petra_skull_fill.inputs.operation = 'fillh'

    skull_petra_pipe.connect(petra_skull_dilate, "out_file",
                             petra_skull_fill, "in_file")

    # petra_skull_erode ####### [okey][json]
    petra_skull_erode = NodeParams(
        interface=ErodeImage(),
        params=parse_key(params, "petra_skull_erode"),
        name="petra_skull_erode")

    skull_petra_pipe.connect(petra_skull_fill, "out_file",
                             petra_skull_erode, "in_file")

    # mesh_petra_skull #######
    mesh_petra_skull = pe.Node(
        interface=niu.Function(input_names=["nii_file"],
                               output_names=["stl_file"],
                               function=wrap_afni_IsoSurface),
        name="mesh_petra_skull")

    skull_petra_pipe.connect(petra_skull_erode, "out_file",
                             mesh_petra_skull, "nii_file")

    if "petra_skull_fov" in params.keys():

        # petra_skull_fov ####### [okey][json]

        petra_skull_fov = NodeParams(
            interface=RobustFOV(),
            params=parse_key(params, "petra_skull_fov"),
            name="petra_skull_fov")

        skull_petra_pipe.connect(petra_skull_erode, "out_file",
                                 petra_skull_fov, "in_file")

        # petra_skull_clean ####### [okey]
        petra_skull_clean = pe.Node(
            interface=niu.Function(input_names=["nii_file"],
                                   output_names=["gcc_nii_file"],
                                   function=keep_gcc),
            name="petra_skull_clean")

        skull_petra_pipe.connect(petra_skull_fov, "out_roi",
                                 petra_skull_clean, "nii_file")

        # mesh_robustpetra_skull #######
        mesh_robustpetra_skull = pe.Node(
            interface=niu.Function(input_names=["nii_file"],
                                   output_names=["stl_file"],
                                   function=wrap_afni_IsoSurface),
            name="mesh_robustpetra_skull")

        skull_petra_pipe.connect(petra_skull_clean, "gcc_nii_file",
                                 mesh_robustpetra_skull, "nii_file")

    # creating outputnode #######
    outputnode = pe.Node(
        niu.IdentityInterface(
            fields=["petra_skull_mask", "petra_skull_stl",
                    "robustpetra_skull_mask", "robustpetra_skull_stl",
                    "petra_head_mask"]),
        name='outputnode')

    skull_petra_pipe.connect(petra_head_erode, "out_file",
                             outputnode, "petra_head_mask")

    skull_petra_pipe.connect(mesh_petra_skull, "stl_file",
                             outputnode, "petra_skull_stl")

    skull_petra_pipe.connect(petra_skull_erode, "out_file",
                             outputnode, "petra_skull_mask")

    if "petra_skull_fov" in params.keys():
        skull_petra_pipe.connect(petra_skull_fov, "out_roi",
                                 outputnode, "robustpetra_skull_mask")

        skull_petra_pipe.connect(mesh_robustpetra_skull, "stl_file",
                                 outputnode, "robustpetra_skull_stl")

    return skull_petra_pipe



#!/usr/bin/env python3
"""
    Non humain primates anatomical segmentation pipeline based ANTS

    Adapted in Nipype from an original pipelin of Kepkee Loh wrapped.

    Description
    --------------
    TODO :/

    Arguments
    -----------
    -data:
        Path to the BIDS directory that contain subjects' MRI data.

    -out:
        Nipype's processing directory.
        It's where all the outputs will be saved.

    -subjects:
        IDs list of subjects to process.

    -ses
        session (leave blank if None)

    -params
        json parameter file; leave blank if None

    Example
    ---------
    python segment_pnh.py -data [PATH_TO_BIDS] -out ../local_tests/ -subjects Elouk

    Requirements
    --------------
    This workflow use:
        - ANTS
        - AFNI
        - FSL
"""

# Authors : David Meunier (david.meunier@univ-amu.fr)
#           Bastien Cagna (bastien.cagna@univ-amu.fr)
#           Kepkee Loh (kepkee.loh@univ-amu.fr)
#           Julien Sein (julien.sein@univ-amu.fr)
#           Adam Sghari (adam.sghari@etu.univ-amu.fr)

import os
import os.path as op

import argparse
import json
import pprint

import nipype.pipeline.engine as pe
import nipype.interfaces.utility as niu

import nipype.interfaces.fsl as fsl

from nipype.interfaces.niftyreg.regutils import RegResample

from macapype.pipelines.full_pipelines import (
    create_full_spm_subpipes,
    create_full_ants_subpipes,
    create_full_T1_ants_subpipes,)

from macapype.utils.utils_bids import (
    create_datasource,
    create_datasource_indiv_params,
    create_datasink)

from macapype.utils.utils_tests import load_test_data, format_template

from macapype.utils.utils_params import update_params

from macapype.utils.misc import show_files, get_first_elem, parse_key

from macapype.pipelines.rename import rename_all_brain_derivatives

from skullTo3d.pipelines.skull_pipe import (
    create_skull_petra_pipe,
    create_skull_ct_pipe,
    create_skull_t1_pipe)

from skullTo3d.pipelines.rename import (
    rename_all_skull_petra_derivatives,
    rename_all_skull_t1_derivatives,
    rename_all_skull_ct_derivatives)


fsl.FSLCommand.set_default_output_type('NIFTI_GZ')
##########################################################################

def create_main_workflow(data_dir, process_dir, soft, species, subjects,
                         sessions, brain_dt, skull_dt, acquisitions,
                         reconstructions, params_file, indiv_params_file,
                         mask_file, template_path, template_files, nprocs,
                         reorient, deriv, pad,
                         wf_name="macapype"):

    # macapype_pipeline
    """ Set up the segmentatiopn pipeline based on ANTS

    Arguments
    ---------
    data_path: pathlike str
        Path to the BIDS directory that contains anatomical images

    out_path: pathlike str
        Path to the ouput directory (will be created if not alredy existing).
        Previous outputs maybe overwritten.

    soft: str
        Indicate which analysis should be launched; so for, only spm and ants
        are accepted; can be extended

    subjects: list of str (optional)
        Subject's IDs to match to BIDS specification (sub-[SUB1], sub-[SUB2]...)

    sessions: list of str (optional)
        Session's IDs to match to BIDS specification (ses-[SES1], ses-[SES2]...)

    acquisitions: list of str (optional)
        Acquisition name to match to BIDS specification (acq-[ACQ1]...)

    reconstructions: list of str (optional)
        Reconstructions name to match to BIDS specification (rec-[ACQ1]...)

    indiv_params_file: path to a JSON file
        JSON file that specify some parameters of the pipeline,
        unique for the subjects/sessions.

    params_file: path to a JSON file
        JSON file that specify some parameters of the pipeline.

    nprocs: integer
        number of processes that will be launched by MultiProc


    Returns
    -------
    workflow: nipype.pipeline.engine.Workflow


    """

    brain_dt = [dt.lower() for dt in brain_dt]

    skull_dt = [dt.lower() for dt in skull_dt]

    print("brain_dt: ", brain_dt)
    print("skull_dt: ", skull_dt)

    soft = soft.lower()

    ssoft = soft.split("_")

    new_ssoft = ssoft.copy()

    if 'test' in ssoft:
        new_ssoft.remove('test')

    if 'prep' in ssoft:
        new_ssoft.remove('prep')

    if 'noseg' in ssoft:
        new_ssoft.remove('noseg')

    if 'native' in ssoft:
        new_ssoft.remove('native')

    if 'template' in ssoft:
        new_ssoft.remove('template')

    soft = "_".join(new_ssoft)

    print("soft: ", soft)
    print("ssoft: ", ssoft)

    # formating args
    data_dir = op.abspath(data_dir)

    process_dir = op.abspath(process_dir)

    try:
        os.makedirs(process_dir)

    except OSError:
        print("process_dir {} already exists".format(process_dir))

    # params
    if params_file is None:

        # species
        if species is not None:

            species = species.lower()

            rep_species = {"marmoset": "marmo",
                           "marmouset": "marmo",
                           "chimpanzee": "chimp"}

            if species in list(rep_species.keys()):
                species = rep_species[species]

            list_species = ["macaque", "marmo", "baboon", "chimp"]

            assert species in list_species, \
                "Error, species {} should in the following list {}".format(
                    species, list_species)

            package_directory = op.dirname(op.abspath(__file__))

            params_file = "{}/params_segment_{}_{}.json".format(
                package_directory, species, soft)

        else:
            print("Error, no -params or no -species was found (one or the \
                other is mandatory)")
            exit(-1)

        print("Using default params file:", params_file)

        params, indiv_params, extra_wf_name = update_params(
            ssoft=ssoft, subjects=subjects, sessions=sessions,
            params_file=params_file, indiv_params_file=indiv_params_file)

    else:

        # format for relative path
        params_file = op.abspath(params_file)

        # params
        assert op.exists(params_file), "Error with file {}".format(
            params_file)

        print("Using orig params file:", params_file)

        params = json.load(open(params_file))

        extra_wf_name = "_orig"
        indiv_params = {}

        # indiv_params
        if indiv_params_file is not None:

            # format for relative path
            indiv_params_file = op.abspath(indiv_params_file)

            assert op.exists(indiv_params_file), "Error with file {}".format(
                indiv_params_file)
            indiv_params = json.load(open(indiv_params_file))

        print("Using indiv_params:", indiv_params)

    # modifying if reorient
    if reorient is not None:
        print("reorient: ", reorient)

        if "skull_petra_pipe" in params.keys():
            params["skull_petra_pipe"]["avg_reorient_pipe"] = {
                "reorient":
                    {"origin": reorient, "deoblique": True}}

        if "short_preparation_pipe" in params.keys():
            params["short_preparation_pipe"]["avg_reorient_pipe"] = {
                "reorient":
                    {"origin": reorient, "deoblique": True}}

    pprint.pprint(params)

    # Workflow
    wf_name += extra_wf_name

    # soft
    wf_name += "_{}".format(soft)

    if 't1' in skull_dt:
        wf_name += "_t1"

    if 'petra' in skull_dt:
        wf_name += "_petra"

    if 'ct' in skull_dt:
        wf_name += "_CT"

    if len(brain_dt) != 0:
        wf_name += "_brain"

    if 't1' in brain_dt:
        wf_name += "_t1"

    if 't2' in brain_dt:
        wf_name += "_t2"

    if mask_file is not None:
        wf_name += "_mask"

    assert "spm" in ssoft or "spm12" in ssoft or "ants" in ssoft, \
        "error with {}, should be among [spm12, spm, ants]".format(ssoft)

    # adding forced space
    if "spm" in ssoft or "spm12" in ssoft:
        if 'native' in ssoft:
            wf_name += "_native"

    elif "ants" in ssoft:
        if "template" in ssoft:
            wf_name += "_template"

    # params_template
    if template_path is not None:

        # format for relative path
        template_path = op.abspath(template_path)

        assert os.path.exists(template_path), "Error, template_path {} do not exists".format(template_path)

        print(template_files)

        params_template = {}

        assert len(template_files) > 1, "Error, template_files unspecified {}".format(template_files)

        template_head = os.path.join(template_path,template_files[0])
        assert os.path.exists(template_head), "Could not find template_head {}".format(template_head)
        params_template["template_head"] = template_head

        template_brain = os.path.join(template_path,template_files[1])
        assert os.path.exists(template_brain), "Could not find template_brain {}".format(template_brain)
        params_template["template_brain"] = template_brain

        if len(template_files) == 3:

            template_seg = os.path.join(template_path,template_files[2])
            assert os.path.exists(template_seg), "Could not find template_seg {}".format(template_seg)
            params_template["template_seg"] = template_seg

        elif len(template_files) == 5:

            template_gm = os.path.join(template_path,template_files[2])
            assert os.path.exists(template_gm), "Could not find template_gm {}".format(template_gm)
            params_template["template_gm"] = template_gm

            template_wm = os.path.join(template_path,template_files[3])
            assert os.path.exists(template_wm), "Could not find template_wm {}".format(template_wm)
            params_template["template_wm"] = template_wm

            template_csf = os.path.join(template_path,template_files[4])
            assert os.path.exists(template_csf), "Could not find template_csf {}".format(template_csf)
            params_template["template_csf"] = template_csf

        else:
            print("Unknown template_files format, should be 3 or 5 files")
            exit(-1)

    else:
        ### use template from params
        assert ("general" in params.keys() and \
            "template_name" in params["general"].keys()), \
                "Error, the params.json should contains a general/template_name"

        template_name = params["general"]["template_name"]

        if "general" in params.keys() and "my_path" in params["general"].keys():
            my_path = params["general"]["my_path"]
        else:
            my_path = ""

        template_dir = load_test_data(template_name, path_to = my_path)
        params_template = format_template(template_dir, template_name)

        if "template_aladin_name" in params["general"].keys():

            template_aladin_name = params["general"]["template_aladin_name"]
            template_aladin_dir = load_test_data(template_aladin_name, path_to = my_path)
            params_template_aladin = format_template(template_aladin_dir, template_aladin_name)

        else:
            params_template_aladin = params_template

        if "template_stereo_name" in params["general"].keys():

            template_stereo_name = params["general"]["template_stereo_name"]
            template_stereo_dir = load_test_data(template_stereo_name, path_to = my_path)
            params_template_stereo = format_template(template_stereo_dir, template_stereo_name)

        else:
            params_template_stereo = params_template

    print (params_template)

    # main_workflow
    main_workflow = pe.Workflow(name= wf_name)

    main_workflow.base_dir = process_dir

    # which soft is used
    if "spm" in ssoft or "spm12" in ssoft:
        if 'native' in ssoft:
            space='native'

        else:
            space='template'

        segment_brain_pipe = create_full_spm_subpipes(
            params_template=params_template,
            params_template_aladin=params_template_aladin,
            params_template_stereo=params_template_stereo,
            params=params, pad=pad, space=space)

    elif "ants" in ssoft:
        if "template" in ssoft:
            space="template"

        else:
            space="native"

        if "t1" in brain_dt and 't2' in brain_dt:
            segment_brain_pipe = create_full_ants_subpipes(
                params_template=params_template,
                params_template_aladin=params_template_aladin,
                params_template_stereo=params_template_stereo,
                params=params, mask_file=mask_file, space=space, pad=pad)

        elif "t1" in brain_dt:
            segment_brain_pipe = create_full_T1_ants_subpipes(
                params_template=params_template,
                params_template_aladin=params_template_aladin,
                params_template_stereo=params_template_stereo,
                params=params, space=space, pad=pad)

    # list of all required outputs
    output_query = {}

    # T1 (mandatory, always added)
    # T2 is optional, if "_T1" is added in the -soft arg
    if 't1' in brain_dt or 't1' in skull_dt :
        output_query['T1'] = {
            "datatype": "anat", "suffix": "T1w",
            "extension": ["nii", ".nii.gz"]}

    if 't2' in brain_dt:
        output_query['T2'] = {
            "datatype": "anat", "suffix": "T2w",
            "extension": ["nii", ".nii.gz"]}

    if 'petra' in skull_dt:
        output_query['PETRA'] = {
            "datatype": "anat", "suffix": "PDw",
            "extension": ["nii", ".nii.gz"]}

    if 'ct' in skull_dt:
        output_query['CT'] = {
            "datatype": "anat", "suffix": "T2star",
            "acquisition": "CT",
            "extension": ["nii", ".nii.gz"]}

    # indiv_params
    if indiv_params:
        print("Using indiv params")

        datasource = create_datasource_indiv_params(
            output_query, data_dir, indiv_params, subjects, sessions,
            acquisitions, reconstructions)

        main_workflow.connect(datasource, "indiv_params",
                              segment_brain_pipe, 'inputnode.indiv_params')
    else:
        datasource = create_datasource(
            output_query, data_dir, subjects,  sessions, acquisitions,
            reconstructions)

    if "t1" in brain_dt:
        main_workflow.connect(datasource, 'T1',
                              segment_brain_pipe, 'inputnode.list_T1')

    if "t2" in brain_dt:
        main_workflow.connect(datasource, 'T2',
                              segment_brain_pipe, 'inputnode.list_T2')

    elif "t1" in brain_dt and "spm" in ssoft:
        # cheating using T2 as T1
        main_workflow.connect(datasource, 'T1',
                              segment_brain_pipe, 'inputnode.list_T2')

    if "petra" in skull_dt and "skull_petra_pipe" in params.keys():
        print("Found skull_petra_pipe")

        skull_petra_pipe = create_skull_petra_pipe(
            params=parse_key(params, "skull_petra_pipe"))

        if "t1" in brain_dt and "t2" in brain_dt:
            # optimal pipeline, use T2
            main_workflow.connect(segment_brain_pipe,
                                  "outputnode.native_T2",
                                  skull_petra_pipe, 'inputnode.native_img')

        elif "t1" in brain_dt:
            main_workflow.connect(segment_brain_pipe,
                                  "outputnode.native_T1",
                                  skull_petra_pipe, 'inputnode.native_img')

        # all remaining connection
        main_workflow.connect(datasource, ('PETRA', show_files),
                              skull_petra_pipe, 'inputnode.petra')

        main_workflow.connect(segment_brain_pipe,
                              "outputnode.stereo_native_T1",
                              skull_petra_pipe, 'inputnode.stereo_native_T1')

        main_workflow.connect(segment_brain_pipe,
                              "outputnode.native_to_stereo_trans",
                              skull_petra_pipe,
                              'inputnode.native_to_stereo_trans')

        if indiv_params:
            main_workflow.connect(datasource, "indiv_params",
                                  skull_petra_pipe, 'inputnode.indiv_params')

        if pad and space == "native":

            # output node
            outputnode = pe.Node(
                niu.IdentityInterface(
                    fields=["native_petra_skull_mask",
                            "native_robustpetra_skull_mask",
                            "native_petra_head_mask"]),
                name='outputnode')

            if "short_preparation_pipe" in params.keys():
                if "crop_T1" in params["short_preparation_pipe"].keys():
                    pass
                else:
                    print("Using reg_aladin transfo to pad skull_mask back")

                    pad_petra_skull_mask = pe.Node(RegResample(inter_val="NN"),
                                                   name="pad_petra_skull_mask")

                    main_workflow.connect(
                        skull_petra_pipe, "outputnode.petra_skull_mask",
                        pad_petra_skull_mask, "flo_file")

                    main_workflow.connect(
                        segment_brain_pipe, "outputnode.native_T1",
                        pad_petra_skull_mask, "ref_file")

                    main_workflow.connect(
                        segment_brain_pipe,
                        "outputnode.cropped_to_native_trans",
                        pad_petra_skull_mask, "trans_file")

                    print("Using reg_aladin transfo \
                        to pad robustpetra_skull_mask back")

                    print("Using reg_aladin transfo to pad head_mask back")

                    pad_petra_head_mask = pe.Node(RegResample(inter_val="NN"),
                                                  name="pad_petra_head_mask")

                    main_workflow.connect(skull_petra_pipe,
                                          "outputnode.petra_head_mask",
                                          pad_petra_head_mask, "flo_file")

                    main_workflow.connect(segment_brain_pipe,
                                          "outputnode.native_T1",
                                          pad_petra_head_mask, "ref_file")

                    main_workflow.connect(segment_brain_pipe,
                                          "outputnode.cropped_to_native_trans",
                                          pad_petra_head_mask, "trans_file")

                    if "petra_skull_fov" in params["skull_petra_pipe"]:
                        pad_robustpetra_skull_mask = pe.Node(
                            RegResample(inter_val="NN"),
                            name="pad_robustpetra_skull_mask")

                        main_workflow.connect(
                            skull_petra_pipe,
                            "outputnode.robustpetra_skull_mask",
                            pad_robustpetra_skull_mask, "flo_file")

                        main_workflow.connect(
                            segment_brain_pipe, "outputnode.native_T1",
                            pad_robustpetra_skull_mask, "ref_file")

                        main_workflow.connect(
                            segment_brain_pipe,
                            "outputnode.cropped_to_native_trans",
                            pad_robustpetra_skull_mask, "trans_file")

                        main_workflow.connect(
                            pad_robustpetra_skull_mask, "out_file",
                            outputnode, "native_robustpetra_skull_mask")

    if "ct" in skull_dt and "skull_ct_pipe" in params.keys():
        print("Found skull_ct_pipe")

        skull_ct_pipe = create_skull_ct_pipe(
            params=parse_key(params, "skull_ct_pipe"))

        main_workflow.connect(datasource, ('CT', get_first_elem),
                              skull_ct_pipe, 'inputnode.ct')

        main_workflow.connect(segment_brain_pipe,
                              "outputnode.native_T1",
                              skull_ct_pipe, 'inputnode.native_T1')

        main_workflow.connect(segment_brain_pipe,
                              "outputnode.native_T2",
                              skull_ct_pipe, 'inputnode.native_T2')

        main_workflow.connect(segment_brain_pipe,
                              "outputnode.stereo_native_T1",
                              skull_ct_pipe, 'inputnode.stereo_native_T1')

        main_workflow.connect(
            segment_brain_pipe, "outputnode.native_to_stereo_trans",
            skull_ct_pipe, 'inputnode.native_to_stereo_trans')

    if 't1' in skull_dt and "skull_t1_pipe" in params.keys():
        print("Found skull_t1_pipe")

        skull_t1_pipe = create_skull_t1_pipe(
            params=parse_key(params, "skull_t1_pipe"))

        print("Using stereo debias T1 for skull_t1_pipe ")

        if "use_debiased_t1" in params["skull_t1_pipe"].keys():
            print("Using stereo debias T1 for skull_t1_pipe ")
            main_workflow.connect(segment_brain_pipe,
                                  "outputnode.stereo_debiased_T1",
                                  skull_t1_pipe, 'inputnode.stereo_native_T1')
        else:

            print("Using stereo native T1 for skull_t1_pipe ")
            main_workflow.connect(segment_brain_pipe,
                                  "outputnode.stereo_native_T1",
                                  skull_t1_pipe, 'inputnode.stereo_native_T1')

        if indiv_params:
            main_workflow.connect(datasource, "indiv_params",
                                  skull_t1_pipe, 'inputnode.indiv_params')

    if deriv:

        datasink_name = os.path.join("derivatives", wf_name)

        if "regex_subs" in params.keys():
            params_regex_subs = params["regex_subs"]
        else:
            params_regex_subs = {}

        if "subs" in params.keys():
            params_subs = params["rsubs"]
        else:
            params_subs = {}

        print(datasource.iterables)

        datasink = create_datasink(iterables=datasource.iterables,
                                   name=datasink_name,
                                   params_subs=params_subs,
                                   params_regex_subs=params_regex_subs)

        datasink.inputs.base_directory = process_dir

        if len(datasource.iterables) == 1:
            pref_deriv = "sub-%(sub)s"
            parse_str = r"sub-(?P<sub>\w*)_.*"
        elif len(datasource.iterables) > 1:
            pref_deriv = "sub-%(sub)s_ses-%(ses)s"
            parse_str = r"sub-(?P<sub>\w*)_ses-(?P<ses>\w*)_.*"

        rename_all_brain_derivatives(
            params, main_workflow, segment_brain_pipe,
            datasink, pref_deriv, parse_str, space, ssoft,
            brain_dt)

        if "petra" in skull_dt and "skull_petra_pipe" in params.keys():
            rename_all_skull_petra_derivatives(
                params, main_workflow, segment_brain_pipe, skull_petra_pipe,
                datasink, pref_deriv, parse_str, space, pad)

            if pad and space == "native":

                # rename petra_skull_mask
                rename_petra_skull_mask = pe.Node(
                    niu.Rename(), name="rename_petra_skull_mask")

                rename_petra_skull_mask.inputs.format_string = \
                    pref_deriv + "_space-{}_desc-petra_skullmask".format(space)
                rename_petra_skull_mask.inputs.parse_string = parse_str
                rename_petra_skull_mask.inputs.keep_ext = True

                main_workflow.connect(
                    pad_petra_head_mask, "out_file",
                    rename_petra_skull_mask, 'in_file')

                main_workflow.connect(
                    rename_petra_skull_mask, 'out_file',
                    datasink, '@petra_skull_mask')

                # rename petra_head_mask
                rename_petra_head_mask = pe.Node(
                    niu.Rename(), name="rename_petra_head_mask")
                rename_petra_head_mask.inputs.format_string = \
                    pref_deriv + "_space-{}_desc-petra_headmask".format(space)
                rename_petra_head_mask.inputs.parse_string = parse_str
                rename_petra_head_mask.inputs.keep_ext = True

                main_workflow.connect(
                    pad_petra_head_mask, "out_file",
                    rename_petra_head_mask, 'in_file')

                main_workflow.connect(
                    rename_petra_head_mask, 'out_file',
                    datasink, '@petra_head_mask')

                if "petra_skull_fov" in params["skull_petra_pipe"]:
                    # rename robustpetra_skull_mask
                    rename_robustpetra_skull_mask = pe.Node(
                        niu.Rename(), name="rename_robustpetra_skull_mask")
                    rename_robustpetra_skull_mask.inputs.format_string = \
                        pref_deriv + \
                        "_space-{}_desc-robustpetra_skullmask".format(
                            space)

                    rename_robustpetra_skull_mask.inputs.parse_string = \
                        parse_str

                    rename_robustpetra_skull_mask.inputs.keep_ext = True

                    main_workflow.connect(
                        pad_robustpetra_skull_mask, "out_file",
                        rename_robustpetra_skull_mask, 'in_file')

                    main_workflow.connect(
                        rename_robustpetra_skull_mask, 'out_file',
                        datasink, '@robustpetra_skull_mask')

        if "ct" in skull_dt and "skull_ct_pipe" in params.keys():
            print("rename ct skull pipe")

            rename_all_skull_ct_derivatives(
                params, main_workflow, segment_brain_pipe, skull_ct_pipe,
                datasink, pref_deriv, parse_str, space, pad)

        if "t1" in skull_dt and "skull_t1_pipe" in params.keys():
            rename_all_skull_t1_derivatives(
                params, main_workflow, segment_brain_pipe, skull_t1_pipe,
                datasink, pref_deriv, parse_str, space, pad)

    main_workflow.write_graph(graph2use="colored")
    main_workflow.config['execution'] = {'remove_unnecessary_outputs': 'false'}

    # saving real params.json
    real_params_file = op.join(process_dir, wf_name, "real_params.json")
    with open(real_params_file, 'w+') as fp:
        json.dump(params, fp)

    if deriv:
        try:
            os.makedirs(op.join(process_dir, datasink_name))
        except OSError:
            print("process_dir {} already exists".format(process_dir))

        real_params_file = op.join(process_dir,
                                   datasink_name, "real_params.json")
        with open(real_params_file, 'w+') as fp:
            json.dump(params, fp)

    if nprocs is None:
        nprocs = 4

    if "test" not in ssoft:
        if "seq" in ssoft or nprocs == 0:
            main_workflow.run()
        else:
            main_workflow.run(plugin='MultiProc',
                              plugin_args={'n_procs': nprocs})


def main():

    # Command line parser
    parser = argparse.ArgumentParser(
        description="PNH segmentation pipeline")

    parser.add_argument("-data", dest="data", type=str, required=True,
                        help="Directory containing MRI data (BIDS)")

    parser.add_argument("-out", dest="out", type=str,
                        help="Output dir", required=True)

    parser.add_argument("-soft", dest="soft", type=str,
                        help="Sofware of analysis (SPM or ANTS are defined)",
                        required=True)

    parser.add_argument("-species", dest="species", type=str,
                        help="Type of PNH to process",
                        required=False)

    parser.add_argument("-subjects", "-sub", dest="sub",
                        type=str, nargs='+', help="Subjects", required=False)

    parser.add_argument("-sessions", "-ses", dest="ses",
                        type=str, nargs='+', help="Sessions", required=False)

    parser.add_argument("-brain_datatypes", "-brain_dt", "-brain",
                        dest="brain_dt", type=str,
                        default=['T1'], nargs='+',
                        help="MRI Brain Datatypes (T1, T2)",
                        required=False)

    parser.add_argument("-skull_datatypes", "-skull_dt", "-skull",
                        dest="skull_dt", type=str,
                        default=['T1'], nargs='+',
                        help="MRI Brain Datatypes (T1, petra, CT)",
                        required=False)

    parser.add_argument("-acquisitions", "-acq", dest="acq", type=str,
                        nargs='+', default=None, help="Acquisitions")

    parser.add_argument("-records", "-rec", dest="rec", type=str, nargs='+',
                        default=None, help="Records")

    parser.add_argument("-params", dest="params_file", type=str,
                        help="Parameters json file", required=False)

    parser.add_argument("-indiv_params", "-indiv", dest="indiv_params_file",
                        type=str, help="Individual parameters json file",
                        required=False)

    parser.add_argument("-mask", dest="mask_file", type=str,
                        help="precomputed mask file", required=False)

    parser.add_argument("-template_path", dest="template_path", type=str,
                        help="specifies user-based template path",
                        required=False)

    parser.add_argument("-template_files", dest="template_files", type=str,
                        nargs="+", help="specifies user-based template files \
                            (3 or 5 are possible options)",
                        required=False)

    parser.add_argument("-nprocs", dest="nprocs", type=int,
                        help="number of processes to allocate", required=False)

    parser.add_argument("-reorient", dest="reorient", type=str,
                        help="reorient initial image", required=False)

    parser.add_argument("-deriv", dest="deriv", action='store_true',
                        help="output derivatives in BIDS orig directory",
                        required=False)

    parser.add_argument("-pad", dest="pad", action='store_true',
                        help="padding mask and seg_mask",
                        required=False)

    args = parser.parse_args()

    # main_workflow
    print("Initialising the pipeline...")
    create_main_workflow(
        data_dir=args.data,
        soft=args.soft,
        process_dir=args.out,
        species=args.species,
        subjects=args.sub,
        sessions=args.ses,
        brain_dt=args.brain_dt,
        skull_dt=args.skull_dt,
        acquisitions=args.acq,
        reconstructions=args.rec,
        params_file=args.params_file,
        indiv_params_file=args.indiv_params_file,
        mask_file=args.mask_file,
        template_path=args.template_path,
        template_files=args.template_files,
        nprocs=args.nprocs,
        reorient=args.reorient,
        deriv=args.deriv,
        pad=args.pad)


if __name__ == '__main__':
    main()



"""
    Gather all full pipelines

"""
import nipype.interfaces.utility as niu
import nipype.pipeline.engine as pe


def rename_all_skull_petra_derivatives(params, main_workflow, segment_pnh_pipe,
                                       skull_petra_pipe, datasink, pref_deriv,
                                       parse_str, space, pad):

    # Rename in skull_petra_pipe
    if "skull_petra_pipe" in params.keys():

        # rename petra_skull_stl
        rename_petra_skull_stl = pe.Node(niu.Rename(),
                                         name="rename_petra_skull_stl")
        rename_petra_skull_stl.inputs.format_string = \
            pref_deriv + "_space-stereo_desc-petra_skullmask"
        rename_petra_skull_stl.inputs.parse_string = parse_str
        rename_petra_skull_stl.inputs.keep_ext = True

        main_workflow.connect(
            skull_petra_pipe, 'outputnode.petra_skull_stl',
            rename_petra_skull_stl, 'in_file')

        main_workflow.connect(
            rename_petra_skull_stl, 'out_file',
            datasink, '@petra_skull_stl')

        # rename stereo_petra_skull_mask
        rename_stereo_petra_skull_mask = pe.Node(
            niu.Rename(), name="rename_stereo_petra_skull_mask")

        rename_stereo_petra_skull_mask.inputs.format_string =\
            pref_deriv + "_space-stereo_desc-petra_skullmask"
        rename_stereo_petra_skull_mask.inputs.parse_string = parse_str
        rename_stereo_petra_skull_mask.inputs.keep_ext = True

        main_workflow.connect(
            skull_petra_pipe, 'outputnode.petra_skull_mask',
            rename_stereo_petra_skull_mask, 'in_file')

        main_workflow.connect(
            rename_stereo_petra_skull_mask, 'out_file',
            datasink, '@stereo_petra_skull_mask')

        # rename stereo_petra_head_mask
        rename_stereo_petra_head_mask = pe.Node(
            niu.Rename(), name="rename_stereo_petra_head_mask")

        rename_stereo_petra_head_mask.inputs.format_string =\
            pref_deriv + "_space-stereo_desc-petra_headmask"
        rename_stereo_petra_head_mask.inputs.parse_string = parse_str
        rename_stereo_petra_head_mask.inputs.keep_ext = True

        main_workflow.connect(
            skull_petra_pipe, 'outputnode.petra_head_mask',
            rename_stereo_petra_head_mask, 'in_file')

        main_workflow.connect(
            rename_stereo_petra_head_mask, 'out_file',
            datasink, '@stereo_petra_head_mask')

        if "petra_skull_fov" in params["skull_petra_pipe"]:

            # rename robustpetra_skull_stl
            rename_robustpetra_skull_stl = pe.Node(
                niu.Rename(), name="rename_robustpetra_skull_stl")

            rename_robustpetra_skull_stl.inputs.format_string = \
                pref_deriv + "_space-stereo_desc-robustpetra_skullmask"
            rename_robustpetra_skull_stl.inputs.parse_string = parse_str
            rename_robustpetra_skull_stl.inputs.keep_ext = True

            main_workflow.connect(
                skull_petra_pipe, 'outputnode.robustpetra_skull_stl',
                rename_robustpetra_skull_stl, 'in_file')

            main_workflow.connect(
                rename_robustpetra_skull_stl, 'out_file',
                datasink, '@robustpetra_skull_stl')

            # rename stereo_robustpetra_skull_mask
            rename_stereo_robustpetra_skull_mask = pe.Node(
                niu.Rename(), name="rename_stereo_robustpetra_skullmask")

            rename_stereo_robustpetra_skull_mask.inputs.format_string = \
                pref_deriv + "_space-stereo_desc-robustpetra_skullmask"

            rename_stereo_robustpetra_skull_mask.inputs.parse_string = \
                parse_str

            rename_stereo_robustpetra_skull_mask.inputs.keep_ext = True

            main_workflow.connect(
                skull_petra_pipe, 'outputnode.robustpetra_skull_mask',
                rename_stereo_robustpetra_skull_mask, 'in_file')

            main_workflow.connect(
                rename_stereo_robustpetra_skull_mask, 'out_file',
                datasink, '@stereo_robustpetra_skullmask')


def rename_all_skull_ct_derivatives(params, main_workflow, segment_pnh_pipe,
                                    skull_ct_pipe, datasink, pref_deriv,
                                    parse_str, space, pad):

    # Rename in skull_ct_pipe
    if "skull_ct_pipe" in params.keys():

        # rename ct_skull_mask
        rename_ct_skull_mask = pe.Node(niu.Rename(),
                                       name="rename_ct_skull_mask")
        rename_ct_skull_mask.inputs.format_string = \
            pref_deriv + "_space-stereo_desc-ct_skullmask"
        rename_ct_skull_mask.inputs.parse_string = parse_str
        rename_ct_skull_mask.inputs.keep_ext = True

        main_workflow.connect(
                skull_ct_pipe, "outputnode.stereo_ct_skull_mask",
                rename_ct_skull_mask, 'in_file')

        main_workflow.connect(
            rename_ct_skull_mask, 'out_file',
            datasink, '@ct_skull_mask')

        # rename ct_skull_stl
        rename_ct_skull_stl = pe.Node(niu.Rename(),
                                      name="rename_ct_skull_stl")
        rename_ct_skull_stl.inputs.format_string = \
            pref_deriv + "_space-stereo_desc-ct_skullmask"
        rename_ct_skull_stl.inputs.parse_string = parse_str
        rename_ct_skull_stl.inputs.keep_ext = True

        main_workflow.connect(
            skull_ct_pipe, 'outputnode.ct_skull_stl',
            rename_ct_skull_stl, 'in_file')

        main_workflow.connect(
            rename_ct_skull_stl, 'out_file',
            datasink, '@ct_skull_stl')


def rename_all_skull_t1_derivatives(params, main_workflow, segment_pnh_pipe,
                                    skull_t1_pipe, datasink, pref_deriv,
                                    parse_str, space, pad):

    # Rename in skull_t1_pipe
    if "skull_t1_pipe" in params.keys():

        # rename t1_skull_mask
        rename_t1_skull_mask = pe.Node(niu.Rename(),
                                       name="rename_t1_skull_mask")
        rename_t1_skull_mask.inputs.format_string = \
            pref_deriv + "_space-stereo_desc-t1_skullmask"
        rename_t1_skull_mask.inputs.parse_string = parse_str
        rename_t1_skull_mask.inputs.keep_ext = True

        main_workflow.connect(
                skull_t1_pipe, "outputnode.t1_skull_mask",
                rename_t1_skull_mask, 'in_file')

        main_workflow.connect(
            rename_t1_skull_mask, 'out_file',
            datasink, '@t1_skull_mask')

        # rename t1_skull_stl
        rename_t1_skull_stl = pe.Node(niu.Rename(),
                                      name="rename_t1_skull_stl")
        rename_t1_skull_stl.inputs.format_string = \
            pref_deriv + "_space-stereo_desc-t1_skullmask"
        rename_t1_skull_stl.inputs.parse_string = parse_str
        rename_t1_skull_stl.inputs.keep_ext = True

        main_workflow.connect(
            skull_t1_pipe, 'outputnode.t1_skull_stl',
            rename_t1_skull_stl, 'in_file')

        main_workflow.connect(
            rename_t1_skull_stl, 'out_file',
            datasink, '@t1_skull_stl')

        # rename t1_head_mask
        rename_t1_head_mask = pe.Node(niu.Rename(),
                                      name="rename_t1_head_mask")
        rename_t1_head_mask.inputs.format_string = \
            pref_deriv + "_space-{}_desc-t1_headmask".format(space)
        rename_t1_head_mask.inputs.parse_string = parse_str
        rename_t1_head_mask.inputs.keep_ext = True

        main_workflow.connect(
            skull_t1_pipe, 'outputnode.t1_head_mask',
            rename_t1_head_mask, 'in_file')

        main_workflow.connect(
            rename_t1_head_mask, 'out_file',
            datasink, '@t1_head_mask')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import re
from setuptools import find_packages, setup

required_packages = [
    "macapype"]

verstr = "unknown"
try:
    verstrline = open('skullTo3d/_version.py', "rt").read()
except EnvironmentError:
    pass  # Okay, there is no version file.

else:
    VSRE = r"^__version__ = ['\"]([^'\"]*)['\"]"
    mo = re.search(VSRE, verstrline, re.M)
    if mo:
        verstr = mo.group(1)
    else:
        raise RuntimeError("unable to find version in yourpackage/_version.py")

print("Will not build conda module")

print("*******************************************************")
print(find_packages())
print("*******************************************************")

setup(
    name="skullTo3d",
    version=verstr,
    packages=find_packages(),
    author="macatools team",
    description="Pipeline for skull extraction for macaque/marmoset",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    license='BSD 3',
    entry_points={
        'console_scripts': ['segment_petra = workflows.segment_petra:main']
        },
    install_requires=required_packages,
    include_package_data=True)
